/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package es.rickyepoderi.bootablejar.rest;

import es.rickyepoderi.bootablejar.MariaDBDatabase;
import es.rickyepoderi.bootablejar.UserEntity;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.ExampleObject;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.parameters.RequestBody;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import java.util.List;
import javax.annotation.Resource;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import javax.transaction.UserTransaction;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 *
 * @author rickyepoderi
 */
@Path("users")
public class UserEndpoint {

    @MariaDBDatabase
    @Inject
    private EntityManager em;

    @Resource
    private UserTransaction utx;

    private void rollbackTransaction() {
        try {
            utx.rollback();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Operation(summary = "List Users",
            tags = {"list"},
            description = "Returns the full list of users in the database",
            responses = {
                @ApiResponse(responseCode = "200", description = "Successfull",
                        content = @Content(array = @ArraySchema(schema = @Schema(implementation = UserEntity.class)))),
                @ApiResponse(responseCode = "500", description = "Server Error")})
    public Response listUsers() {
        try {
            utx.begin();
            TypedQuery<UserEntity> query = em.createQuery("SELECT user FROM UserEntity user", UserEntity.class);
            List<UserEntity> results = query.getResultList();
            utx.commit();
            return Response.ok(results).build();
        } catch (Exception e) {
            e.printStackTrace();
            rollbackTransaction();
            return Response.serverError().build();
        }
    }

    @GET
    @Path("{id:[0-9]+}")
    @Produces(MediaType.APPLICATION_JSON)
    @Operation(summary = "Get User",
            tags = {"get"},
            description = "Returns a single user by id",
            responses = {
                @ApiResponse(responseCode = "200", description = "Successfull", content = @Content(schema = @Schema(implementation = UserEntity.class))),
                @ApiResponse(responseCode = "500", description = "Server Error"),
                @ApiResponse(responseCode = "404", description = "User Not Found")})
    public Response getUser(@Parameter(description = "User ID") @PathParam("id") long id) {
        try {
            utx.begin();
            UserEntity user = em.find(UserEntity.class, id);
            utx.commit();
            if (user == null) {
                return Response.status(Response.Status.NOT_FOUND).build();
            }
            return Response.ok(user, MediaType.APPLICATION_JSON).build();
        } catch (Exception e) {
            e.printStackTrace();
            rollbackTransaction();
            return Response.serverError().build();
        }
    }

    @DELETE
    @Path("{id:[0-9]+}")
    @Operation(summary = "Delete User",
            tags = {"delete"},
            description = "Deletes a user by id",
            responses = {
                @ApiResponse(responseCode = "200", description = "Successfull"),
                @ApiResponse(responseCode = "500", description = "Server Error"),
                @ApiResponse(responseCode = "404", description = "User Not Found")})
    public Response deleteUser(@Parameter(description = "User ID") @PathParam("id") long id) {
        try {
            utx.begin();
            Query query = em.createQuery("DELETE FROM UserEntity user WHERE user.id = :id");
            query.setParameter("id", id);
            int rows = query.executeUpdate();
            utx.commit();
            if (rows == 0) {
                return Response.status(Response.Status.NOT_FOUND).build();
            }
            return Response.ok().build();
        } catch (Exception e) {
            e.printStackTrace();
            rollbackTransaction();
            return Response.serverError().build();
        }
    }

    @PUT
    @Produces(MediaType.APPLICATION_JSON)
    @Consumes(MediaType.APPLICATION_JSON)
    @Operation(summary = "Create new user",
            tags = {"create"},
            description = "Creates a new user",
            responses = {
                @ApiResponse(responseCode = "200", description = "Successfull", content = @Content(schema = @Schema(implementation = UserEntity.class))),
                @ApiResponse(responseCode = "500", description = "Server Error"),
                @ApiResponse(responseCode = "400", description = "Bad Request")})
    public Response createUser(@RequestBody(
            description = "Details of the User to be created",
            required = true,
            content = @Content(
                    schema = @Schema(implementation = UserEntity.class),
                    mediaType = MediaType.APPLICATION_JSON,
                    examples = {
                        @ExampleObject(
                                name = "UserEntity example",
                                value = "{\"firstName\": \"John\", \"lastName\": \"Dow\"}",
                                summary = "User example")})) UserEntity user) {
        if (user == null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("Empty user").build();
        }
        if (user.getId() != null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("ID cannot be send for create").build();
        }
        if (user.getFirstName() == null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("Empty FIRSTNAME").build();
        }
        if (user.getLastName() == null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("Empty LASTNAME").build();
        }
        try {
            utx.begin();
            em.persist(user);
            utx.commit();
            return Response.ok(user).build();
        } catch (Exception e) {
            e.printStackTrace();
            rollbackTransaction();
            return Response.serverError().build();
        }
    }

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    @Consumes(MediaType.APPLICATION_JSON)
    @Operation(summary = "Update user",
            tags = {"update"},
            description = "Updates a user",
            responses = {
                @ApiResponse(responseCode = "200", description = "Successfull"),
                @ApiResponse(responseCode = "500", description = "Server Error"),
                @ApiResponse(responseCode = "400", description = "Bad Request"),
                @ApiResponse(responseCode = "404", description = "User Not Found")})
    public Response updateUser(@RequestBody(
            description = "Details of the User to be updated",
            required = true,
            content = @Content(
                    schema = @Schema(implementation = UserEntity.class),
                    mediaType = MediaType.APPLICATION_JSON,
                    examples = {
                        @ExampleObject(
                                name = "UserEntity example",
                                value = "{\"id\": 1, \"firstName\": \"John\", \"lastName\": \"Dow\"}",
                                summary = "User example")})) UserEntity user) {
        if (user == null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("Empty user").build();
        }
        if (user.getId() == null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("Empty ID").build();
        }
        if (user.getFirstName() == null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("Empty FIRSTNAME").build();
        }
        if (user.getLastName() == null) {
            return Response.status(Response.Status.BAD_REQUEST).entity("Empty LASTNAME").build();
        }
        try {
            utx.begin();
            UserEntity oldUser = em.find(UserEntity.class, user.getId());
            if (oldUser != null) {
                oldUser.setFirstName(user.getFirstName());
                oldUser.setLastName(user.getLastName());
            }
            utx.commit();
            if (oldUser == null) {
                return Response.status(Response.Status.NOT_FOUND).build();
            } else {
                return Response.ok().build();
            }
        } catch (Exception e) {
            e.printStackTrace();
            rollbackTransaction();
            return Response.serverError().build();
        }
    }
}
